# Generated by Django 4.0.2 on 2022-04-11 09:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djwebdapp', '0003_transaction_txgroup_alter_transaction_hash_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='transaction',
            options={'ordering': ['-created_at']},
        ),
    ]
