# Generated by Django 3.1.14 on 2022-02-20 09:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('djwebdapp', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EthereumTransaction',
            fields=[
                ('transaction_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djwebdapp.transaction')),
                ('abi', models.JSONField(blank=True, default=dict, help_text='Smart contract ABI, if this is a smart contract', null=True)),
                ('input', models.TextField(blank=True, help_text='Input hex string if any', null=True)),
                ('bytecode', models.TextField(blank=True, help_text='Contract bytecode if this is a smart contract to deploy', null=True)),
                ('contract', models.ForeignKey(blank=True, help_text='Smart contract, appliable to method call', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='call_set', to='djwebdapp_ethereum.ethereumtransaction')),
            ],
            bases=('djwebdapp.transaction',),
        ),
    ]
